@@ -7,8 +7,7 @@ import seaborn as sns
 import pandas as pd
 
 
-from sklearn.feature_selection import (f_regression,
-                                       mutual_info_regression,
+from sklearn.feature_selection import (mutual_info_regression,
                                        mutual_info_classif,
                                        f_classif)
 from sklearn.impute import SimpleImputer
@@ -23,13 +22,20 @@ from .utils import (_check_X_target_col, _get_n_top, _make_subplots,
                     find_pretty_grid, _find_scatter_plots_classification,
                     class_hists, discrete_scatter, mosaic_plot,
                     _find_inliers, pairplot, _get_scatter_alpha,
-                    _get_scatter_size)
+                    _get_scatter_size, _find_categorical_for_regression,
+                    _prune_categories)
+from .sankey import plot_sankey
+
 from warnings import warn
 
 
-def plot_regression_continuous(X, target_col, types=None,
+def plot_regression_continuous(X, *, target_col, types=None,
                                scatter_alpha='auto', scatter_size='auto',
-                               drop_outliers=True, **kwargs):
+                               drop_outliers=True, correlation="spearman",
+                               prune_correlations_threshold=0.95,
+                               find_scatter_categoricals=True,
+                               jitter_ordinal=True,
+                               **kwargs):
     """Plots for continuous features in regression.
 
     Creates plots of all the continuous features vs the target.
@@ -49,8 +55,26 @@ def plot_regression_continuous(X, target_col, types=None,
     scatter_size : float, default='auto'
         Marker size for scatter plots. 'auto' is dirty hacks.
     drop_outliers : bool, default=True
-        Whether to drop outliers when plotting.
+        Whether to drop outliers (in the target column) when plotting.
+    correlation : str, default="spearman"
+        Correlation to use for ranking plots, passed to
+        ``pd.DataFrame.corrwith``.
+        Valid values are `'pearson'`, `'kendall'`, `'spearman'`.
+    jitter_ordinal : bool, default=True
+        Whether to add jitter, i.e. apply noise, to ordinal features, to reduce overlap.
+    prune_correlations_threshold : float, default=.95
+        Whether to prune highly correlated features from the plot.
+        Set to 0 to disable pruning.
+    find_scatter_categoricals : boolean, default=True
+        Whether to find categorical features to use as hue in scatter plots.
+
     """
+    X = X.copy()
+    X.columns = X.columns.astype(str)
+    if types is not None:
+        types = types.copy()
+        types.index = types.index.astype(str)
+
     types = _check_X_target_col(X, target_col, types, task="regression")
 
     if np.isnan(X[target_col]).any():
@@ -68,34 +92,62 @@ def plot_regression_continuous(X, target_col, types=None,
     if features.shape[1] == 0:
         return
 
-    show_top = _get_n_top(features, "continuous")
-
     target = X[target_col]
-    # HACK we should drop them per column before feeding them into f_regression
-    # FIXME
-    features_imp = SimpleImputer().fit_transform(features)
-    f, p = f_regression(features_imp, target)
-    top_k = np.argsort(f)[-show_top:][::-1]
-    # we could do better lol
-    fig, axes = _make_subplots(n_plots=show_top)
+
+    correlations = features.corrwith(target, method=correlation)
+    corrs = correlations.abs().sort_values().dropna().index[::-1]
+    if prune_correlations_threshold > 0:
+        top_k = []
+        for col in corrs:
+            corr = features[top_k].corrwith(features[col], method="spearman")
+            corr = corr.abs()
+            if not len(top_k) or corr.max() < prune_correlations_threshold:
+                top_k.append(col)
+            else:
+                warn(f"Not plotting highly correlated ({corr.max()})"
+                     f" feature {col}. Set prune_correlations_threshold=0 to"
+                     f" keep.")
+            if len(top_k) > 20:
+                warn("Showing only top 10 continuous features.")
+                top_k = top_k[:10]
+                break
+
+    else:
+        show_top = _get_n_top(features, "continuous")
+
+        top_k = corrs[:show_top]
+
+    for cat_col in X.columns[types.categorical]:
+        X[cat_col] = _prune_categories(X[cat_col])
+
+    if find_scatter_categoricals:
+        best_categorical = _find_categorical_for_regression(
+            X, types, target_col, top_cont=top_k, random_state=None)
+    else:
+        best_categorical = []
+
+    fig, axes = _make_subplots(n_plots=len(top_k))
 
     # FIXME this could be a function or maybe using seaborn
     plt.suptitle("Continuous Feature vs Target")
     scatter_alpha = _get_scatter_alpha(scatter_alpha, X[target_col])
     scatter_size = _get_scatter_size(scatter_size, X[target_col])
-    for i, (col_idx, ax) in enumerate(zip(top_k, axes.ravel())):
+    for i, (col, ax) in enumerate(zip(top_k, axes.ravel())):
         if i % axes.shape[1] == 0:
             ax.set_ylabel(_shortname(target_col))
-        col = features.columns[col_idx]
-        if drop_outliers:
-            inliers = _find_inliers(features.loc[:, col])
-            ax.scatter(features.loc[inliers, col], target[inliers],
-                       alpha=scatter_alpha, s=scatter_size)
-        else:
-            ax.scatter(features.loc[:, col], target,
-                       alpha=scatter_alpha, s=scatter_size)
-        ax.set_xlabel(_shortname(col))
-        ax.set_title("F={:.2E}".format(f[col_idx]))
+        c = X.loc[:, best_categorical[col]] if len(best_categorical) and best_categorical[col] is not None else None
+        values = features.loc[:, col]
+        col_name = _shortname(col, maxlen=50)
+        # FIXME we should know low-card-int here but we overwrote it unfortunately in plot
+        if jitter_ordinal and X[col].nunique() < 20:
+            values = values + np.random.normal(0, .1, size=len(features))
+            col_name = col_name + " (jittered)"
+        discrete_scatter(values, target,
+                         c=c,
+                         clip_outliers=drop_outliers, alpha=scatter_alpha,
+                         s=scatter_size, ax=ax, legend=True, **kwargs)
+        ax.set_xlabel(col_name)
+        ax.set_title("F={:.2E}".format(correlations[col]))
 
     for j in range(i + 1, axes.size):
         # turn off axis if we didn't fill last row
@@ -103,7 +155,8 @@ def plot_regression_continuous(X, target_col, types=None,
     return axes
 
 
-def plot_regression_categorical(X, target_col, types=None, **kwargs):
+def plot_regression_categorical(
+        X, *, target_col, types=None, drop_outliers=True, **kwargs):
     """Plots for categorical features in regression.
 
     Creates box plots of target distribution for important categorical
@@ -121,9 +174,14 @@ def plot_regression_categorical(X, target_col, types=None, **kwargs):
     types : dataframe of types, optional
         Output of detect_types on X. Can be used to avoid recomputing the
         types.
+    drop_outliers : bool, default=True
+        Whether to drop outliers (in the target column) when plotting.
     """
     types = _check_X_target_col(X, target_col, types, task="regression")
 
+    if drop_outliers:
+        inliers = _find_inliers(X.loc[:, target_col])
+        X = X.loc[inliers, :]
     # drop nans from target column
     if np.isnan(X[target_col]).any():
         X = X.dropna(subset=[target_col])
@@ -160,7 +218,7 @@ def plot_regression_categorical(X, target_col, types=None, **kwargs):
         sns.boxplot(x=target_col, y=col, data=X_new, order=order, ax=ax)
         ax.set_title("F={:.2E}".format(f[col_ind]))
         # shorten long ticks and labels
-        _short_tick_names(ax)
+        _short_tick_names(ax, label_length=row_height * 12)
 
     for j in range(i + 1, axes.size):
         # turn off axis if we didn't fill last row
@@ -168,12 +226,13 @@ def plot_regression_categorical(X, target_col, types=None, **kwargs):
     return axes
 
 
-def plot_classification_continuous(X, target_col, types=None, hue_order=None,
-                                   scatter_alpha='auto', scatter_size="auto",
-                                   univariate_plot='histogram',
-                                   drop_outliers=True, plot_pairwise=True,
-                                   top_k_interactions=10, random_state=None,
-                                   **kwargs):
+def plot_classification_continuous(
+        X, *, target_col, types=None, hue_order=None,
+        scatter_alpha='auto', scatter_size="auto",
+        univariate_plot='histogram',
+        drop_outliers=True, plot_pairwise=True,
+        top_k_interactions=10, random_state=None,
+        **kwargs):
     """Plots for continuous features in classification.
 
     Selects important continuous features according to F statistics.
@@ -395,7 +454,7 @@ def _plot_univariate_classification(features, features_imp, target,
     return f
 
 
-def plot_classification_categorical(X, target_col, types=None, kind='auto',
+def plot_classification_categorical(X, *, target_col, types=None, kind='auto',
                                     hue_order=None, **kwargs):
     """Plots for categorical features in classification.
 
@@ -450,33 +509,32 @@ def plot_classification_categorical(X, target_col, types=None, kind='auto',
         ordinal_encoded, target,
         discrete_features=np.ones(X.shape[1], dtype=bool))
     top_k = np.argsort(f)[-show_top:][::-1]
+
+    if kind == 'sankey':
+        return plot_sankey(X, target_col,
+                           figure=kwargs.get('figure', None))
     # large number of categories -> taller plot
     row_height = 3 if features.nunique().max() <= 5 else 5
     fig, axes = _make_subplots(n_plots=show_top, row_height=row_height)
     plt.suptitle("Categorical Features vs Target", y=1.02)
     for i, (col_ind, ax) in enumerate(zip(top_k, axes.ravel())):
         col = features.columns[col_ind]
+        # how many categories make up at least 1% of data:
+        n_cats = (X[col].value_counts() / len(X) > 0.01).sum()
+        n_cats = np.minimum(n_cats, 20)
+        X_new = _prune_category_make_X(X, col, target_col,
+                                       max_categories=n_cats)
         if kind == 'proportion':
-            X_new = _prune_category_make_X(X, col, target_col)
-
             df = (X_new.groupby(col)[target_col]
                   .value_counts(normalize=True)
                   .unstack()
                   .sort_values(by=target[0]))  # hacky way to get a class name
             df.plot(kind='barh', stacked='True', ax=ax, legend=i == 0)
-            ax.set_title(col)
             ax.set_ylabel(None)
         elif kind == 'mosaic':
-            # how many categories make up at least 1% of data:
-            n_cats = (X[col].value_counts() / len(X) > 0.01).sum()
-            n_cats = np.minimum(n_cats, 20)
-            X_new = _prune_category_make_X(X, col, target_col,
-                                           max_categories=n_cats)
             mosaic_plot(X_new, col, target_col, ax=ax, legend=i == 0)
-            ax.set_title(col)
         elif kind == 'count':
             X_new = _prune_category_make_X(X, col, target_col)
-
             # absolute counts
             # FIXME show f value
             # FIXME shorten titles?
@@ -491,6 +549,7 @@ def plot_classification_categorical(X, target_col, types=None, kind='auto',
         else:
             raise ValueError("Unknown plot kind {}".format(kind))
         _short_tick_names(ax)
+        ax.set_title(col)
 
     for j in range(i + 1, axes.size):
         # turn off axis if we didn't fill last row
@@ -498,14 +557,17 @@ def plot_classification_categorical(X, target_col, types=None, kind='auto',
 
 
 def plot(X, y=None, target_col=None, type_hints=None, scatter_alpha='auto',
-         scatter_size='auto', verbose=10, plot_pairwise=True,
-         **kwargs):
+         scatter_size='auto', drop_outliers=True, verbose=10,
+         plot_pairwise=True, **kwargs):
     """Automatic plots for classification and regression.
 
     Determines whether the target is categorical or continuous and plots the
     target distribution. Then calls the relevant plotting functions
     accordingly.
 
+    See the functions in the "see also" section for more parameters that
+    can be passed as kwargs.
+
 
     Parameters
     ----------
@@ -528,6 +590,8 @@ def plot(X, y=None, target_col=None, type_hints=None, scatter_alpha='auto',
         These can be somewhat expensive to compute.
     verbose : int, default=10
         Controls the verbosity (output).
+    drop_outliers : bool, default=True
+        Whether to drop outliers in the target column for regression.
 
     See also
     --------
@@ -575,23 +639,33 @@ def plot(X, y=None, target_col=None, type_hints=None, scatter_alpha='auto',
     if types.continuous[target_col]:
         print("Target looks like regression")
         # FIXME we might be overwriting the original dataframe here?
-        X[target_col] = X[target_col].astype(np.float)
+        X[target_col] = X[target_col].astype(float)
         # regression
         # make sure we include the target column in X
         # even though it's not categorical
+
+        if drop_outliers:
+            inliers = _find_inliers(X.loc[:, target_col])
+            n_outliers = len(X) - inliers.sum()
+            if n_outliers > 0:
+                warn(f"Discarding {n_outliers} outliers in target column.",
+                     UserWarning)
+                X = X.loc[inliers, :]
+        _, ax = plt.subplots()
         plt.hist(X[target_col], bins='auto')
         plt.xlabel(_shortname(target_col))
         plt.ylabel("frequency")
         plt.title("Target distribution")
+        res.append(ax)
         scatter_alpha = _get_scatter_alpha(scatter_alpha, X[target_col])
         scatter_size = _get_scatter_size(scatter_size, X[target_col])
 
         res.append(plot_regression_continuous(
-            X, target_col, types=types,
+            X, target_col=target_col, types=types,
             scatter_alpha=scatter_alpha,
             scatter_size=scatter_size, **kwargs))
         res.append(plot_regression_categorical(
-            X, target_col, types=types, **kwargs))
+            X, target_col=target_col, types=types, **kwargs))
     else:
         print("Target looks like classification")
         # regression
@@ -614,10 +688,10 @@ def plot(X, y=None, target_col=None, type_hints=None, scatter_alpha='auto',
                   "Current visualizations are quite useless for"
                   " this many classes. Try slicing the data.")
         res.append(plot_classification_continuous(
-            X, target_col, types=types, hue_order=counts.index,
+            X, target_col=target_col, types=types, hue_order=counts.index,
             scatter_alpha=scatter_alpha, scatter_size=scatter_size,
             plot_pairwise=plot_pairwise, **kwargs))
         res.append(plot_classification_categorical(
-            X, target_col, types=types,
+            X, target_col=target_col, types=types,
             hue_order=counts.index, **kwargs))
     return res
