@@ -284,10 +284,18 @@ def _get_n_top(features, name):
     return show_top
 
 
-def _prune_categories(series, max_categories=10):
-    small_categories = series.value_counts()[max_categories:].index
-    res = series.cat.remove_categories(small_categories)
-    res = res.cat.add_categories(['dabl_other']).fillna("dabl_other")
+def _prune_categories(series, max_categories=10, min_samples=None, replace=True):
+    if not pd.api.types.is_categorical_dtype(series):
+        series = pd.Series(series).astype("category")
+    category_counts = series.value_counts()
+    drop = pd.Series(False, index=category_counts.index)
+    drop.iloc[max_categories:] = True
+    drop[category_counts < .01 * category_counts.median()] = True
+    if min_samples is not None:
+        drop[category_counts < min_samples] = True
+    res = series.cat.remove_categories(drop[drop].index)
+    if res.isnull().any() and replace:
+        res = res.cat.add_categories(['dabl_other']).fillna("dabl_other")
     return res
 
 
@@ -302,6 +310,7 @@ def _prune_category_make_X(X, col, target_col, max_categories=20):
         # keep only top 10 categories if there are more than 20
         col_values = _prune_categories(col_values,
                                        max_categories=min(10, max_categories))
+    col_values = col_values.cat.remove_unused_categories()
     X_new[col] = col_values
     return X_new
 
@@ -370,7 +379,10 @@ def _short_tick_names(ax, label_length=20, ticklabel_length=10):
         Length of xlabel and ylabel
     ticklabel_length : int, default=10
         Length of each label in xticklabels and yticklabels
+
     """
+    ax.set_yticks(ax.get_yticks().tolist())
+    ax.set_xticks(ax.get_xticks().tolist())
     ax.set_xticklabels(
         [_shortname(t.get_text(), maxlen=ticklabel_length)
          for t in ax.get_xticklabels()]
@@ -383,27 +395,62 @@ def _short_tick_names(ax, label_length=20, ticklabel_length=10):
     ax.set_ylabel(_shortname(ax.get_ylabel(), maxlen=label_length))
 
 
+def _score_triple(X, cont1, cont2, cat, random_state):
+    # score how well you can predict a category from two continuous features
+    # assume this tree is simple enough so not be able to overfit in 2d
+    # so we don't bother with train/test split
+    if hasattr(X, "loc"):
+        # we passed column names in a pandas dataframe
+        X = X.dropna(subset=[cont1, cont2, cat])
+        this_X = X.loc[:, [cont1, cont2]]
+        target = X.loc[:, cat].cat.codes
+    else:
+        # we passed indices into a numpy array,
+        # cat is an integer array
+        this_X = X[:, [cont1, cont2]]
+        target = cat
+    n_splits = 3
+    target = _prune_categories(target, replace=False, min_samples=n_splits)
+    if target.isna().any():
+        mask = target.isna()
+        target = target[~mask]
+        this_X = this_X[~mask]
+    # limit to 2000 training points for speed?
+    train_size = min(2000, int(.9 * this_X.shape[0]))
+    if train_size == 0:
+        return 0
+    cv = StratifiedShuffleSplit(n_splits=n_splits, train_size=train_size,
+                                random_state=random_state)
+    tree = DecisionTreeClassifier(max_leaf_nodes=8)
+    return np.mean(cross_val_score(
+        tree, this_X, target, cv=cv, scoring='recall_macro'))
+
+
+def _find_categorical_for_regression(
+        X, types, target_col, top_cont, random_state=None):
+    categorical_features = X.columns[types.categorical]
+    scores = [(cont, cat, _score_triple(
+                X, target_col, cont, cat, random_state=random_state))
+              for cont, cat in itertools.product(
+                top_cont, categorical_features)]
+    scores_df = pd.DataFrame(scores, columns=['cont', 'cat', 'score'])
+    scores = scores_df.pivot(index='cat', columns='cont', values='score')
+    if len(scores) == 0:
+        return []
+    best_categorical = scores.idxmax()
+    # Scores are macro recall, < .5 means uninformative, so we don't plot it
+    best_categorical[scores.max() < .5] = None
+    return best_categorical
+
+
 def _find_scatter_plots_classification(X, target, how_many=3,
                                        random_state=None):
     # input is continuous
     # look at all pairs of features, find most promising ones
-    # dummy = DummyClassifier(strategy='prior').fit(X, target)
-    # baseline_score = recall_score(target, dummy.predict(X), average='macro')
-    scores = []
     # converting to int here might save some time
     _, target = np.unique(target, return_inverse=True)
-    # limit to 2000 training points for speed?
-    train_size = min(2000, int(.9 * X.shape[0]))
-    cv = StratifiedShuffleSplit(n_splits=3, train_size=train_size,
-                                random_state=random_state)
-    for i, j in itertools.combinations(np.arange(X.shape[1]), 2):
-        this_X = X[:, [i, j]]
-        # assume this tree is simple enough so not be able to overfit in 2d
-        # so we don't bother with train/test split
-        tree = DecisionTreeClassifier(max_leaf_nodes=8)
-        scores.append((i, j, np.mean(cross_val_score(
-            tree, this_X, target, cv=cv, scoring='recall_macro'))))
-
+    scores = [(i, j, _score_triple(X, i, j, target, random_state=random_state))
+              for i, j in itertools.combinations(np.arange(X.shape[1]), 2)]
     scores = pd.DataFrame(scores, columns=['feature0', 'feature1', 'score'])
     top = scores.sort_values(by='score').iloc[-how_many:][::-1]
     return top
@@ -466,7 +513,9 @@ def discrete_scatter(x, y, c, unique_c=None, legend='first',
     if legend == "first":
         legend = (ax.get_subplotspec().get_geometry()[2] == 1)
     if unique_c is None:
-        unique_c = np.unique(c)
+        unique_c = c.unique() if c is not None else []
+    if len(unique_c) == 0:
+        ax.scatter(x, y, s=s, alpha=alpha, **kwargs)
     for i in unique_c:
         mask = c == i
         ax.scatter(x[mask], y[mask], label=i, s=s, alpha=alpha, **kwargs)
@@ -478,11 +527,11 @@ def discrete_scatter(x, y, c, unique_c=None, legend='first',
         ax.set_xlim(max(x_low, xlims[0]), min(x_high, xlims[1]))
         ax.set_ylim(max(y_low, ylims[0]), min(y_high, ylims[1]))
 
-    if legend:
+    if legend and c is not None:
         props = {}
         if len(unique_c) > 15:
             props['size'] = 6
-        legend = ax.legend(prop=props)
+        legend = ax.legend(prop=props, title=getattr(c, 'name', None))
         for handle in legend.legendHandles:
             handle.set_alpha(1)
             handle.set_sizes((100,))
@@ -543,16 +592,13 @@ def class_hists(data, column, target, bins="auto", ax=None, legend=True,
     bottom = counts.max().max() * 1.1
     for i, name in enumerate(counts.columns):
         if ordinal:
-            ax.bar(range(counts.shape[0]), counts[name], width=.9,
-                   bottom=bottom * i, tick_label=counts.index, linewidth=2,
-                   edgecolor='k', label=name)
-            xmin, xmax = 0 - .5, counts.shape[0] - .5
+            ax.stairs(counts[name] + bottom * i, range(counts.shape[0] + 1),
+                      baseline=bottom * i,
+                      edgecolor='k', label=name, fill=True)
         else:
-            ax.bar(bin_edges[:-1], counts[name], bottom=bottom * i, label=name,
-                   align='edge', width=(bin_edges[1] - bin_edges[0]) * .9)
-            xmin, xmax = bin_edges[0], bin_edges[-1]
-        ax.hlines(bottom * i, xmin=xmin, xmax=xmax,
-                  linewidth=1)
+            ax.stairs(counts[name] + bottom * i, bin_edges,
+                      baseline=bottom * i,
+                      label=name, edgecolor='k', fill=True)
     if legend:
         ax.legend()
     ax.set_yticks(())
