@@ -17,7 +17,11 @@ _MIXED_TYPE_WARNINGS = {}
 
 
 def _float_matching(X_col, return_safe_col=False):
-    is_floaty = X_col.str.match(_FLOAT_REGEX)
+    try:
+        is_floaty = X_col.str.match(_FLOAT_REGEX)
+    except TypeError as e:
+        warn(f"Error in string parsing:{str(e)}")
+        return pd.Series(False, index=X_col.index)
     # things that weren't strings
     not_strings = is_floaty.isna()
     if not_strings.any():
@@ -28,7 +32,7 @@ def _float_matching(X_col, return_safe_col=False):
             rest.astype(float)
             is_floaty[not_strings] = True
             all_castable = True
-        except ValueError:
+        except (ValueError, TypeError):
             pass
         if not all_castable:
             if X_col.name not in _MIXED_TYPE_WARNINGS:
@@ -114,19 +118,20 @@ class DirtyFloatCleaner(BaseEstimator, TransformerMixin):
                 result.append(X_new_col)
                 continue
             cats = pd.DataFrame(0, index=X.index,
-                                columns=enc.get_feature_names([str(col)]))
+                                columns=enc.get_feature_names_out([str(col)]))
             if nofloats.any():
                 cats.loc[nofloats, :] = enc.transform(pd.DataFrame(
                     X_col[nofloats]))
+                cats = cats.copy()
             cats["{}_dabl_continuous".format(col)] = X_new_col
             result.append(cats)
         return pd.concat(result, axis=1)
 
-    def get_feature_names(self, input_features=None):
+    def get_feature_names_out(self, input_features=None):
         feature_names = []
         for col in self.columns_:
             enc = self.encoders_[col]
-            feature_names.extend(enc.get_feature_names([str(col)]))
+            feature_names.extend(enc.get_feature_names_out([str(col)]))
             feature_names.append("{}_dabl_continuous".format(col))
         return feature_names
 
@@ -208,7 +213,7 @@ def _float_col_is_int(series):
     return True
 
 
-_FLOAT_TYPES = ['floating', 'mixed-interger-float', 'decimal']
+_FLOAT_TYPES = ['floating', 'mixed-integer-float', 'decimal']
 _INTEGER_TYPES = ['integer']
 _DATE_TYPES = ['datetime64', 'datetime', 'date',
                'timedelta64', 'timedelta', 'time', 'period']
@@ -216,6 +221,9 @@ _DATE_TYPES = ['datetime64', 'datetime', 'date',
 _OBJECT_TYPES = ['string', 'bytes', 'mixed', 'mixed-integer']
 _CATEGORICAL_TYPES = ['categorical', 'boolean']
 
+USELESS_TYPES = {'missing', 'unique', 'constant', 'near_constant',
+                 'index', 'mixed-unhashable', 'list', 'dict'}
+
 
 def _get_max_cat_cardinality(n_samples, max_cat_cardinality="auto"):
     if max_cat_cardinality == "auto":
@@ -233,11 +241,11 @@ def _type_detection_int(series, max_cat_cardinality='auto'):
         if series.iloc[0] == 0:
             if (series == np.arange(len(series))).all():
                 # definitely an index
-                return 'useless'
+                return 'index'
         elif series.iloc[0] == 1:
             if (series == np.arange(1, len(series) + 1)).all():
                 # definitely an index
-                return 'useless'
+                return 'index'
     if n_distinct_values > _get_max_cat_cardinality(len(series),
                                                     max_cat_cardinality):
         return 'continuous'
@@ -275,20 +283,27 @@ def _type_detection_object(series, *, dirty_float_threshold,
 def detect_type_series(series, *, dirty_float_threshold=0.9,
                        max_cat_cardinality='auto',
                        near_constant_threshold=0.95, target_col=None):
-    n_distinct_values = series.nunique()
+    try:
+        n_distinct_values = series.nunique()
+    except TypeError:
+        types = series.dropna().map(lambda x: type(x).__name__)
+        if types.nunique() == 1:
+            return types.iloc[0]
+        else:
+            return 'mixed-unhashable'
     if series.isna().mean() > 0.99:
-        return 'useless'
+        return "missing"
     # infer near-constant-values
     # fast-pass if n_distinct_values is high
     count = series.count()
 
     if n_distinct_values == 1:
-        return 'useless'
+        return 'constant'
 
     if (n_distinct_values < (1 - near_constant_threshold) * count
             and series.name != target_col):
         if series.value_counts().max() > near_constant_threshold * count:
-            return 'useless'
+            return 'near_constant'
     if n_distinct_values == 2:
         return 'categorical'
 
@@ -374,13 +389,13 @@ def detect_types(X, type_hints=None, max_cat_cardinality='auto',
     if type_hints is None:
         type_hints = dict()
 
-    n_samples, _ = X.shape
-
     types_series = X.apply(lambda col: detect_type_series(
         col, max_cat_cardinality=max_cat_cardinality,
         near_constant_threshold=near_constant_threshold,
         target_col=target_col, dirty_float_threshold=dirty_float_threshold))
 
+    types_series[types_series.isin(USELESS_TYPES)] = 'useless'
+
     for t in type_hints:
         if t in X.columns:
             types_series[t] = type_hints[t]
@@ -453,6 +468,8 @@ def clean(X, type_hints=None, return_types=False,
     types = types.loc[~types.useless, :]
     for col in types.index[types.categorical]:
         X[col] = X[col].astype('category', copy=False)
+    for col in types.index[types.continuous]:
+        X[col] = X[col].astype(float, copy=False)
 
     if types['dirty_float'].any():
         # don't use ColumnTransformer that can't return dataframe yet
@@ -629,7 +646,7 @@ class EasyPreprocessor(BaseEstimator, TransformerMixin):
         # Return the transformer
         return self
 
-    def get_feature_names(self):
+    def get_feature_names_out(self):
         # this can go soon hopefully
         feature_names = []
         for name, trans, cols in self.ct_.transformers_:
@@ -652,7 +669,7 @@ class EasyPreprocessor(BaseEstimator, TransformerMixin):
                 ohe_cols = ohe_cols.to_list()
                 ohe_cols.extend(added_cols)
 
-                feature_names.extend(ohe.get_feature_names(ohe_cols))
+                feature_names.extend(ohe.get_feature_names_out(ohe_cols))
             elif name == "remainder":
                 assert trans == "drop"
             elif name == "dirty_float":
