@@ -11,7 +11,7 @@ from sklearn.model_selection import train_test_split
 from sklearn.pipeline import make_pipeline
 from sklearn.linear_model import LogisticRegression
 
-from dabl.preprocessing import (detect_types, EasyPreprocessor,
+from dabl.preprocessing import (USELESS_TYPES, detect_types, EasyPreprocessor,
                                 DirtyFloatCleaner, clean, _FLOAT_REGEX,
                                 _float_matching, detect_type_series)
 from dabl.utils import data_df_from_bunch
@@ -57,7 +57,7 @@ def test_duplicate_columns():
 
 def test_duplicate_index():
     X = X_cat.copy()
-    X.index = np.ones(len(X), np.int)
+    X.index = np.ones(len(X), np.int64)
     assert not X.index.is_unique
     with pytest.raises(ValueError):
         detect_types(X)
@@ -96,6 +96,9 @@ def test_continuous_castable():
     types = detect_types(X)
     assert types.continuous['a']
 
+    X_new = clean(X)
+    assert X_new.dtypes['a'] == np.float64
+
 
 # @pytest.mark.parametrize("null_object", [np.nan, None]) FIXME in sklearn
 @pytest.mark.parametrize("null_object", [np.nan])
@@ -122,7 +125,8 @@ def test_dirty_float_single_warning():
         X = pd.DataFrame({'dirty3': dirty3})
         clean(X)
 
-        assert len(w) == 1
+        mixed_type_errors = sum(("Mixed type" in str(w2) for w2 in w))
+        assert mixed_type_errors == 1
 
 
 def test_detect_types():
@@ -138,10 +142,10 @@ def test_detect_types():
          'binary_int': np.random.randint(0, 2, size=100),
          'categorical_int': np.random.randint(0, 4, size=100),
          'low_card_float_int': np.random.randint(
-             0, 4, size=100).astype(np.float),
+             0, 4, size=100).astype(float),
          'low_card_float': np.random.randint(
-             0, 4, size=100).astype(np.float) + 0.1,
-         'binary_float': np.random.randint(0, 2, size=100).astype(np.float),
+             0, 4, size=100).astype(float) + 0.1,
+         'binary_float': np.random.randint(0, 2, size=100).astype(float),
          'cont_int': np.repeat(np.arange(50), 2),
          'unique_string': [random_str() for i in range(100)],
          'continuous_float': np.random.normal(size=100),
@@ -151,8 +155,14 @@ def test_detect_types():
          'near_constant_float': near_constant_float,
          'index_0_based': np.arange(100),
          'index_1_based': np.arange(1, 101),
-         'index_shuffled': np.random.permutation(100)
+         'index_shuffled': np.random.permutation(100),
+         'some_lists': [[random_str() for i in range(3)] for j in range(100)],
+         'some_dicts': [{random_str(): random_str() for i in range(3)}
+                        for j in range(100)]
          })
+    df_all.loc[0, 'some_lists'] = np.NaN
+    df_all['mixed_structured'] = df_all['some_lists']
+    df_all.loc[::2, 'mixed_structured'] = df_all.loc[::2, 'some_dicts']
     res = detect_types(df_all)
     types = res.T.idxmax()
     assert types['categorical_string'] == 'categorical'
@@ -174,6 +184,9 @@ def test_detect_types():
     assert types['index_1_based'] == 'useless'
     # Not detecting a shuffled index right now :-/
     assert types['index_shuffled'] == 'continuous'
+    assert types['some_lists'] == 'useless'
+    assert types['some_dicts'] == 'useless'
+    assert types['mixed_structured'] == 'useless'
 
     res = detect_types(X_cat)
     assert len(res) == 3
@@ -189,7 +202,21 @@ def test_detect_types():
 
     for col in df_all.columns:
         t = detect_type_series(df_all[col])
-        assert t == types[col]
+        if t in USELESS_TYPES:
+            assert types[col] == 'useless'
+        else:
+            assert t == types[col]
+
+    assert detect_type_series(df_all['constant_nan']) == 'missing'
+    assert detect_type_series(df_all['constant_string']) == 'constant'
+    assert detect_type_series(df_all['constant_float']) == 'constant'
+    assert detect_type_series(df_all['near_constant_float']) == 'near_constant'
+    assert detect_type_series(df_all['index_0_based']) == 'index'
+    assert detect_type_series(df_all['index_1_based']) == 'index'
+    assert detect_type_series(df_all['some_lists']) == 'list'
+    assert detect_type_series(df_all['some_dicts']) == 'dict'
+    assert detect_type_series(df_all['mixed_structured']) == 'mixed-unhashable'
+
 
 def test_detect_low_cardinality_int():
     df_all = pd.DataFrame(
@@ -276,10 +303,9 @@ def test_transform_dirty_float():
     res = dfc.transform(dirty)
     # TODO test for new values in test etc
     assert res.shape == (100, 3)
-    assert (res.dtypes == float).all()
     assert res.a_column_missing.sum() == 9
     assert res.a_column_garbage.sum() == 1
-    assert (dfc.get_feature_names() == res.columns).all()
+    assert (dfc.get_feature_names_out() == res.columns).all()
 
 
 @pytest.mark.parametrize(
@@ -302,7 +328,7 @@ def test_type_hints(type_hints):
         assert types.T.idxmax()[k] == v
         # conversion successful
         if v == 'continuous':
-            assert X_clean[k].dtype == np.float
+            assert X_clean[k].dtype == float
         elif v == 'categorical':
             assert X_clean[k].dtype == 'category'
 
@@ -374,16 +400,31 @@ def test_titanic_feature_names():
         'boat_6', 'boat_7', 'boat_8', 'boat_8 10', 'boat_9', 'boat_?',
         'boat_A', 'boat_B', 'boat_C', 'boat_C D', 'boat_D', 'age_?_0.0',
         'age_?_1.0', 'body_?_0.0', 'body_?_1.0']
-    assert ep.get_feature_names() == expected_names
+    try:
+        assert ep.get_feature_names_out() == expected_names
+    except AssertionError:
+        # OHE uses int in newer versions
+        expected_names[57] = 'age_?_0'
+        expected_names[58] = 'age_?_1'
+        expected_names[59] = 'body_?_0'
+        expected_names[60] = 'body_?_1'
+        assert ep.get_feature_names_out() == expected_names
 
     # without clean
     X = ep.fit_transform(titanic.drop('survived', axis=1))
     # FIXME can't do that yet
-    # assert ep.get_feature_names() == expected_names_no_clean
+    # assert ep.get_feature_names_out() == expected_names_no_clean
 
     assert not np.isnan(X).any()
 
 
+def test_type_detection_bytes():
+    import os
+    df = pd.DataFrame({'a': [os.urandom(5) for io in range(100)]})
+    types = detect_types(df)
+    assert types.T.idxmax()['a'] == 'free_string'
+
+
 def test_digits_type_hints():
     data_bunch = load_digits()
 
@@ -420,10 +461,12 @@ def test_simple_preprocessor_imputed_features():
     ep.fit(data)
 
     expected_names = ['A_0', 'A_1', 'A_2', 'A_imputed_False', 'A_imputed_True']
-    assert ep.get_feature_names() == expected_names
+    assert ep.get_feature_names_out() == expected_names
 
 
 def test_dirty_float_target_regression():
+    import matplotlib
+    matplotlib.use("agg")
     titanic_data = load_titanic()
     data = pd.DataFrame({'one': np.repeat(np.arange(50), 2)})
     dirty = make_dirty_float()
@@ -436,7 +479,7 @@ def test_dirty_float_target_regression():
         plot(data, target_col="target")
 
     # check if works for non dirty_float targets
-    plot(titanic_data, 'survived')
+    plot(titanic_data, target_col='survived')
 
 
 def test_string_types_detection():
